# frozen_string_literal: true

require "packman/github"
require "packman/github/repository/version"

require "git"
require "octokit"

module Packman
  module Github
    class Repository
      attr_reader :org, :name

      def initialize(org, name, github_options: {})
        @org = org
        @name = name
        @config = Github.config(github_options)
      end

      def full_name
        "#{org}/#{name}"
      end

      def sync_directory
        "#{repos_directory}/#{full_name}"
      end

      def cloned?
        Dir.exist?(sync_directory) && Git.open(sync_directory)
      rescue ArgumentError
        FileUtils.rm_rf(sync_directory)
        false
      end

      def sync!
        if cloned?
          git = Git.open(sync_directory)
          git.checkout('master')
          git.pull
        else
          Git.clone(clone_url, sync_directory)
        end
      end

      def push!
        git = Git.open(sync_directory)
        git.branch(branch_name).checkout
        git.commit_all('update')
        git.push('origin', branch_name)
      end

      def pull_request!
        pr = octokit.create_pull_request(
          full_name,
          'master',
          branch_name,
          "bundle update #{Time.now.strftime('%Y-%m-%d %H:%M:%S')}",
          "Pull requested from [PackmanBot](https://github.com/packmanbot)."
        )
        pr[:html_url]

      rescue Octokit::UnprocessableEntity => e
        puts e
      end

      private

      def repos_directory
        ENV['PACKMAN_REPOS_DIR'] || File.realpath("#{__dir__}/../../../repos")
      end

      def clone_url
        "https://#{@config.token}:x-oauth-basic@#{@config.domain}/#{full_name}"
      end

      def branch_name
        @branch_name ||= "update-#{Time.now.strftime('%Y%m%d%H%M%S')}"
      end

      def octokit
        @octokit ||= Octokit::Client.new(
          api_endpoint: @config.api_endpoint,
          access_token: @config.token
        )
      end
    end
  end
end
